/* Copyright (C) 2002  Martin J. Muench <mjm@codito.de> */

#include "config.h"
#include "chat.h"
#include "sendicmp.h"
#include "crypt.h"

int chat(const char *host, const char *nick, int icmpcode, char *key)
{
	int rawsock, numActiveFDs, af=0, auth=0, icmpseq=4;
	unsigned char buf[1024];
	char hostnick[13], decrypted[768], text[CHARPERLINE], *payload, *ptr;
	struct sockaddr_in saddr;
	FILE *fp=stdin;

	fd_set readset;
	FD_ZERO(&readset);

	if ((rawsock=socket(AF_INET,SOCK_RAW,IPPROTO_ICMP)) < 0)
	{
		perror("Socket");
		return 0;
	}

	if (setsockopt(rawsock,IPPROTO_IP,IP_HDRINCL,&af,sizeof(af)) < 0)
	{
		close(rawsock);
		return 0;
	}

	fprintf(stderr, ">>> Connecting to %s...\n", host);
	if(!sendpacket(&rawsock, encrypt_buf(key, PING), host, icmpcode, icmpseq))
	{
	 	close(rawsock);
		perror("Sendpacket failed");
		return 0;
	}

	/* Start main chat */
	for(;;)
	{

	  FD_SET(rawsock, &readset);
	  FD_SET(0, &readset);

	  if((numActiveFDs = select(rawsock + 1, &readset, NULL, NULL, NULL)) < 0)
	  {
		close(rawsock);
		return 0;
	  }

	  /* Check for data on raw socket */
	  if (FD_ISSET(rawsock, &readset))
	  {

	    memset(buf, 0, sizeof(buf));
	    if (recvfrom(rawsock,buf,1024,0,(struct sockaddr *)&saddr,(socklen_t *)&af) < 0)
	    {
	    	close(rawsock);
	    	return 0;
	    }

	    ip=(struct ip *)buf;
	    icmp=(struct icmp *)&buf[ip->ip_hl<<2];

	    if (!strcmp(host,inet_ntoa(ip->ip_src)))
	    {
		if(auth > 0)
			if(icmp->icmp_seq == icmpseq)
				continue;

		payload = (char*)(buf + sizeof(struct ip) + sizeof(struct icmp));

		memset(decrypted, 0, sizeof(decrypted));
		strncpy(decrypted, decrypt_buf(key, payload), sizeof(decrypted));
		if(decrypted == NULL)
			break;

		  if(!strcmp(decrypted,PING) && (!auth || auth == 1))
		  {

			/* Send authentication string 1 */
			icmpseq=5;
			if(!sendpacket(&rawsock, encrypt_buf(key, PONG), host, icmpcode, icmpseq))
			{
		  		close(rawsock);
				return 0;
			}
			auth=1;

			if(!sendpacket(&rawsock, encrypt_buf(key, "/hnick %s", nick), host, icmpcode, icmpseq))
			{
				close(rawsock);
				return 0;
			}
		  }

		  else if(!strcmp(decrypted,PONG) && (!auth || auth==1))
		  {

			if (!auth)
			{
			   auth=2;
			   icmpseq=4;
			   if(!sendpacket(&rawsock, encrypt_buf(key, PONG), host, icmpcode, icmpseq))
			   {
			  	close(rawsock);
				return 0;
			   }

			   if(!sendpacket(&rawsock, encrypt_buf(key, "/hnick %s", nick), host, icmpcode, icmpseq))
			   {
			  	close(rawsock);
				return 0;
			   }
			}
			auth=2;
		  }

		  else if(decrypted[0] == '/')
		  {

			if(!strcmp(decrypted,"/quit"))
				fprintf(stderr, ">>> %s has left the chat\n", hostnick);

			else if((ptr = strstr(decrypted, "/hnick")) != NULL)
			{
 				ptr += strlen("/hnick ");
				snprintf(hostnick, sizeof(hostnick), "%s", ptr);
				fprintf(stderr, ">>> %s is connected\n", hostnick);
			}

			else if((ptr = strstr(decrypted, "/setname")) != NULL)
			{
				ptr += strlen("/setname ");
				ptr[strlen(ptr)-1]='\0';
				fprintf(stderr, ">>> %s is now known as %s\n", hostnick, ptr);
				snprintf(hostnick, sizeof(hostnick), "%s", ptr);
			}
		  }

		  else if (auth)
			fprintf(stderr, "%s: %s", hostnick, decrypted);
	    }
          }

	  /* Check for data on stdin */
	  else if (FD_ISSET(fileno(fp), &readset))
	  {

    		memset(text, 0, sizeof(text));
		if (!read(0, text, sizeof(text) - 1))
		{
			close(rawsock);
			return 0;
		}

		else
		{

			/* The professional ;-) command detection */
 			if(text[0] == '/')
 			{

	 		  if(!strncmp(text,"/quit", 5))
			  {
			     if(!sendpacket(&rawsock, encrypt_buf(key, "/quit"), host, icmpcode, icmpseq))
			     {
				close(rawsock);
				return 0;
			     }
			     return 1;
		          }
			  else if(!strncmp(text,"/setname", 8))
			  {
			     if(!sendpacket(&rawsock, encrypt_buf(key, text), host, icmpcode, icmpseq))
			     {
			     	close(rawsock);
			     	return 0;
			     }
			  }
			  else if((ptr = strstr(text,"/setcode")) != NULL)
			  {
				ptr += strlen("/setcode ");
				ptr[strlen(ptr)-1]='\0';
				icmpcode=atoi(ptr);
				fprintf(stderr, ">>> You are now using icmpcode %d\n", icmpcode);

			  }
			  else if(!strncmp(text,"/help", 5))
			  	fprintf(stderr, "%s", HELP);
		    	  else
				fprintf(stderr, ">>> Unknown command: %s\n", text);
		  	}

			else
	  		{
			  if(!sendpacket(&rawsock, encrypt_buf(key, text), host, icmpcode, icmpseq))
			  {
	  			close(rawsock);
				return 0;
	  		  }
			}

		}
	  }
	}
  return 1;
}

