/* Copyright (C) 2002  Martin J. Muench <mjm@codito.de> */

#include "config.h"
#include "setup.h"
#include "chat.h"
#include "crypt.h"
#include "wrap.h"

int main(int argc, char **argv)
{

	char host[256], nick[13], key[56];
	int icmpcode;
	struct hostent *hostent;

	if (argc < 4)
		usage(argv[0]);

	if (getuid() != 0)
	{
		fprintf(stderr, "Error: you need to be root!\n");
		exit(-1);
	}

	/* Argv[1] = Hostname */
	if ((hostent = Gethostbyname(argv[1])) == NULL) 
	{
		perror("Error looking up hostname");
		exit(-1);
	}
	strncpy(host,inet_ntoa(*(struct in_addr*)hostent->h_addr_list[0]),sizeof(host)-1);
	
	#ifdef DEBUG
	debug("Host is: %s", host);
	#endif

	/* Argv[2] = ICMP_Code */
	icmpcode = atoi(argv[2]);

	#ifdef DEBUG
	debug("ICMP_Code is: %d", icmpcode);
	#endif

	/* Argv[3] = Nickname */
	memset(nick, 0, sizeof(nick));
	strncpy(nick, argv[3], sizeof(nick)-1);

	#ifdef DEBUG
	debug("Nickname is: %s", nick);
	#endif

	/* Key */
	memset(key, 0, sizeof(key));
	strncpy(key, Getpass(), sizeof(key));
	if(key == NULL)
	{
		fprintf(stderr, "Error getting password\n");
		exit(-1);
	}

	#ifdef DEBUG
	debug("Key is: %s", key);
	#endif

	/* Catch signals */
	signal(SIGINT, Sigint);

	#ifdef DEBUG
	debug("Set signal");
	#endif

	fprintf(stderr, "%s\n", BANNER);

	if(!chat(host, nick, icmpcode, key))
	{
		perror(">>> Aborting, an error occured");
		exit(-1);
	}
	
	fprintf(stderr, ">>> ICMP Chat exits now...\n");
	exit(0);
}

void usage(const char *name) 
{
	fprintf(stderr, "\nICMP-Chat %s\n-------------\n\n"            		  \
			"Usage: %s <host> <icmp_code> <nickname>\n\n"             \
			"For a list of possible ICMP codes look at the README\n"  \
			"\n", VERSION, name);
	exit(-1);
}
